#!/usr/bin/perl 

#use diagnostics;
use warnings;
use strict;
use Getopt::Std;
use Net::SMTP::SSL;
use DBI;
use Config::IniFiles;
use Net::IP qw(:PROC);


sub format_mail {
	my $txt = $_[0];
	my $dat =$_[1];
	#my $ip = new Net::IP ($dat->{device_device_ip_address }) or die (Net::IP::Error());
	my $sip = $dat->{device_ip_address};
	$txt =~ s/\%node.name/$dat->{device_name}/gi;
	$txt =~ s/\%node.ip/$sip/gi;
	$txt =~ s/\%node.description/$dat->{device_description}/gi;
	$txt =~ s/\%service.name/$dat->{service_name}/gi;
	$txt =~ s/\%service.parameter/$dat->{service_parameter}/gi;
	$txt =~ s/\%error.type/$dat->{error_type}/gi;
	$txt =~ s/\%error.name/$dat->{error_name}/gi;
	$txt =~ s/\%error.time/$dat->{happen_timestamp}/gi;
	$txt =~ s/\%error.message/$dat->{happen_message}/gi;
	$txt =~ s/\\n/\n/gi;
	return $txt;
#Noeud : %node.name : %node.ip \n Service: %service   ETAT: %error.type Heure: %error.time \n %error.message 
}

my %opts;	
my $config="/etc/esia/esia.conf";
#test des paramètres
if(!getopts( "e:t:c:h", \%opts ))
{
  print "Usage : send_mail.pl -e <id_error> -t <to> -c <esia config file>\n";
  exit(1);
}
if($opts{'h'})
{
  print "Usage : send_mail.pl -e <id_error> -t <to> -c <esia config file>\n"; exit(1); 
}
if(!$opts{'e'})
{
   print "no error is defined\n"; exit(1); 
}
if($opts{'c'})
{
  $config=$opts{'c'};
}
if(!$opts{'t'})
{
  print "mail to not defined\n"; exit(1); 
}
#chargement du fichier de configuration
my $cfg = new Config::IniFiles( -file => $config  )or die "esia config file not found (default: /etc/esia/esia.conf)";
#Connexion à la base de donnée
my $db = DBI -> connect($cfg->val( 'POSTGRESQL', 'PGSQL_connect' ),$cfg->val( 'POSTGRESQL', 'PGSQL_username' ),$cfg->val( 'POSTGRESQL', 'PGSQL_pwd' )) or die "Connection DB Failed";
#connexion au serveur SMTP
my $to = $opts{'t'};
my $host;
my $port=25;
my $smtp; 
my $user;
my $password;
my $from;
my $message;
my $subject;
my $query = $db->prepare("SELECT config_key,config_value
                          FROM esia_configuration
                          WHERE config_key LIKE 'MAIL_%';");	
$query->execute();
while (my @data = $query->fetchrow_array ) {
	if($data[0] eq 'MAIL_SMTP')
	{	
		$host = $data[1];
	}
	elsif($data[0] eq 'MAIL_SMTP_PORT')
	{
		$port = $data[1];
	} 
	elsif($data[0] eq 'MAIL_USER')
	{
		$user = $data[1];
	}
	elsif($data[0] eq 'MAIL_PASSWORD')
	{
		$password = $data[1];
	}
	elsif($data[0] eq 'MAIL_FROM')
	{
		$from = $data[1];
	}
	elsif($data[0] eq 'MAIL_FORMAT_MESSAGE')
	{
		$message = $data[1];
	}
	elsif($data[0] eq 'MAIL_FORMAT_SUBJECT')
	{
		$subject = $data[1];
	}
}
print"$host:$port $user:$password $from:\n";
#connexion sécurisée
if($port=='465')
{
  $smtp = Net::SMTP::SSL->new(Host =>$host,Hello=>$from ,Port=> $port,Timeout => 120) or die "Couldn't connect to server";
  $smtp->auth($user ,$password) or die "Could not authenticate $! \n quit";
}
else
{
  #connexion classique 
  $smtp = Net::SMTP->new(Host =>$host,Hello=>$from ,Port=> $port,Timeout => 120) or die "Couldn't connect to server";
  if($user)
  {
    $smtp->auth($user,$password) or die "Could not authenticate $! \n quit";
  }
}
#récupération des informations sur l'erreur
$query = $db->prepare("SELECT device_name,device_ip_address,device_description,device_ip_version,device_description,error_name,error_type,happen_timestamp,happen_message,service_name,service_parameter 
                          FROM esia_error_happen
                          JOIN esia_device ON happen_device_id = device_id
                          JOIN esia_errors ON happen_error_id= error_id 
                          JOIN esia_services ON happen_service_id= service_id
                          WHERE happen_id=?");	
$query->execute($opts{'e'});
my $data = $query->fetchrow_hashref;
if($data)
{
	$subject = format_mail($subject,$data);	
	$message = format_mail($message,$data);	
 	$smtp->mail("$from");
	$smtp->to( split (/,/,$to));
	$smtp->data();
	$smtp->datasend("To: $to\n");
	$smtp->datasend("From: $from\n");
	$smtp->datasend("X-Mailer: Perl Sendmail \n");
	$smtp->datasend("Subject: $subject\n");
	$smtp->datasend("\n");
	$smtp->datasend("$message\n");
	$smtp->dataend();
	$smtp->quit();
}
$query->finish;
$db->disconnect; 
exit(0);
